-- Minetest mod: creepers
-- (c) Kai Gerd Müller
-- See README.txt for licensing and other information.
local creepers_cid_data = {}
minetest.register_node("creepers:air", {
	drawtype = "airlike",
	tiles = {},
	light_source = 14,
	groups = {not_in_creative_inventory = 1},
	drop = '',
	walkable = false,
	buildable_to = true,
	pointable = false,
	damage_per_second = 4,
	on_blast = function() end,
	on_timer = function(pos)minetest.remove_node(pos)end
})
minetest.after(0, function()
	for name, def in pairs(minetest.registered_nodes) do
		if name == "fire:basic_flame" then
			tntafire = true
		elseif name == "default:coalblock" then
			tntacoal = true
		end
		if def.groups then
		r = 1
		r=hardness(def,r,"cracky")
		r=hardness(def,r,"choppy")
		r=hardness(def,r,"crumbly")
		r=hardness(def,r,"oddly_breakable_by_hand")
		r=hardness(def,r,"snappy")
		r=hardness(def,r,"dig_immediate")
		if def.groups.level then
		if type(def.groups.level) == "number" then
			r = r/math.max(1,def.groups.level)
		end
		end
		r = (1-1/(r+1))*100
		else
		r = 0
		end
		if def.liquidtype == "source" or def.liquidtype == "flowing" then
			r = 100
		end
		creepers_cid_data[minetest.get_content_id(name)] = {
			name = name,
			drops = def.drop,
			flammable = def.groups.flammable,
			hardness = r,
			igniter = def.groups.igniter,
			on_blast = def.on_blast,
			mesecon = mesecon,
			def = def
		}
	end
	cid_fire = minetest.get_content_id("fire:basic_flame")
	cid_coal = minetest.get_content_id("default:coalblock")
	cid_air = minetest.get_content_id("air")
end)



minetest.register_entity("creepers:mover",{
	hp_max		= 1000000,
	physical	= true,
	collisionbox	= { -1/2, -1/2, -1/2, 1/2, 1/2, 1/2 },
	textures = {"none.png"},
	timer		= 0,
	physical_state	= true,
	on_activate = function(sf,dt)
		sf.object:set_armor_groups({immortal =1})
	end,
	on_step = function( sf, dt )
		sf.timer = sf.timer + dt
		if sf.timer > 1.5 then
			sf.object:remove()
		end
	end
})
function dir(pos,radius,dmg)
	local objs = minetest.get_objects_inside_radius(pos, radius)
	for i, obj in ipairs(objs) do
		obj:set_hp(obj:get_hp()-dmg)
	end
end
function gass(pos,radius,textures,damage)
	minetest.add_particlespawner({
		amount = radius*radius*radius*80,
		time = 60,
		minpos = vector.subtract(pos, radius/2),
		maxpos = vector.add(pos, radius/2),
		minvel = {x=-1, y=-1, z=-1},
		maxvel = {x=1,  y=1,  z=1},
		minacc = vector.new(),
		maxacc = vector.new(),
		minexptime = 2,
		maxexptime = 8,
		minsize = 4,
		maxsize = 6,
		texture = textures .. ".png",
	})
	for i = 0,60 do
		minetest.after(i,dir,pos,radius,damage)
	end
end
function hardness(def,n,value)
	if def.groups[value] then
		if type(def.groups[value]) == "number" then
			n = n*math.max(1,def.groups[value])
		end
	end
	return n
end
local function drop_unified_inventory(pos,drops,force)
	for _, drop in pairs(drops) do
	if type(creepers_cid_data[drop.content].name) == "string" and(type(creepers_cid_data[drop.content].drops)=="string"or creepers_cid_data[drop.content].drops == nil)then
	drop =(creepers_cid_data[drop.content].drops or creepers_cid_data[drop.content].name) .. " " .. tostring(drop.number)
	obj = minetest.add_item(pos,drop)
	if obj then
		obj:setvelocity({x=math.random(-force,force), y=math.random(0,force), z=math.random(-force,force)})
	end
	end
	end
end

function creepers_explodeadvanced(pos,radius,damage)
	local drops = {}
	local vm = VoxelManip()
	local p1 = vector.subtract(pos, radius)
	local p2 = vector.add(pos, radius)
	local minp, maxp = vm:read_from_map(p1, p2)
	local a = VoxelArea:new({MinEdge = minp, MaxEdge = maxp})
	local data = vm:get_data()
	local c_air = minetest.get_content_id("air")
	local distsq = radius*radius
	local p = {x = 0,y = 0,z = 0}
	for z = -radius, radius do
	for y = -radius, radius do
	for x = -radius, radius do
		if (x * x) + (y * y) + (z * z) <= distsq then
			p = {x = pos.x + x, y = pos.y + y, z = pos.z + z}
			local i = a:index(p.x,p.y,p.z)
			local cid = data[i]
			if cid ~= c_air then
				lossy = creepers_cid_data[cid].hardness
				if lossy == nil or math.random(1, 100)*vector.distance(pos, p)/radius<lossy then
	local def = creepers_cid_data[cid]
	if def and def.on_blast then
		def.on_blast(a:position(i), 1)
	elseif def and def.flammable then
		if math.random(1,5) == 1 and tntafire then
			data[i]=cid_fire
		elseif math.random(1,50) == 1 and tntacoal then
			data[i]=cid_coal
		else
			data[i]=cid_air
		end
	elseif def then
		data[i]=cid_air
		if drops[cid] then
			drops[cid]={content = cid,number=drops[cid].number+1}
		else
			drops[cid]={content = cid,number = 1}
		end
	else
		data[i]=cid_air
	end
					end
			end
		end
	end
	end
	end
	vm:set_data(data)
	vm:calc_lighting()
	vm:write_to_map()
	vm:update_liquids()
	vm:update_map()
	minetest.sound_play("explosion", {pos = pos, gain = radius*10, max_hear_distance = radius*10})
	radius = radius * 2
	local objs = minetest.get_objects_inside_radius(pos, radius)
	for _, obj in pairs(objs) do
		local obj_pos = obj:getpos()
		local obj_vel = obj:getvelocity()
		local dist = math.max(1, vector.distance(pos, obj_pos))
		if obj_vel == nil then
			obj_vel = {x = 0,y = 0,z = 0}
		end
		vel = vector.add(vector.multiply(vector.normalize(vector.direction(pos, obj_pos)), 1000/dist), obj_vel)
		if obj:is_player() == true then
			local emp = minetest.add_entity( obj_pos, 'creepers:mover' )
			obj:set_attach(emp, "", {x = 0, y = 0, z = 0}, {x = 0, y = 0, z = 0})
			emp:setvelocity({x=vel.x, y=vel.y, z=vel.z})
			emp:setacceleration({x=-vel.x/5, y=-10, z=-vel.z/5})
		else
			obj:setvelocity(vel)
		end
		obj:set_hp(obj:get_hp() - ((4 / dist) * radius*damage))
	end
	radius = radius / 2
	for i = 1,12 do
		minetest.add_particlespawner({
			amount = 256,
			time = 5,
			minpos = vector.subtract(pos, radius / 2),
			maxpos = vector.add(pos, radius / 2),
			minvel = {x=-20, y=-20, z=-20},
			maxvel = {x=20,  y=20,  z=20},
			minacc = vector.new(),
			maxacc = vector.new(),
			minexptime = 0.25,
			maxexptime = 0.5,
			minsize = 8,
			maxsize = 16,
			texture = tostring(i) .. ".png",
		})
	end
	drop_unified_inventory(pos,drops,radius*2)
end
--[[#########################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
#############################################################################################################################################################################################################
]]
local creepers_cave_boni = 5
local creepers_charging_constant = 3
local creepers_spawn_constant = 500
local function len(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end
function drop(dt)
	for _,i in pairs(dt) do
	db("SSS")
	db(i.chanche)
	if math.random(1,i.chanche) == 1 then
	db(i.chanche)
	local obj = minetest.add_item(pos, ItemStack(i.name .. math.floor(math.random(i.min, i.max))))
	if obj then
	obj:setvelocity({x=math.random(-1,1), y=1, z=math.random(-1,1)})
	db("earned sth")
	end
	end
	end
end
function dropondie(self,itemstack)
	if self.object:get_hp() > 0 then return end
	self.object:remove()
	local pos = self.object:getpos()
		minetest.add_particlespawner({
			amount = 256,
			time = 1,
			minpos = vector.subtract(pos,{x=0.5,y=0,z=0.5}),
			maxpos = vector.add(pos,{x=0.5,y=2,z=0.5}),
			minvel = {x=-1, y=-2, z=-1},
			maxvel = {x=1,  y=1,  z=1},
			minacc = vector.new(),
			maxacc = vector.new(),
			minexptime = 0.5,
			maxexptime = 1,
			minsize = 1,
			maxsize = 2,
			physical = true,
			texture = "6.png",
		})
		minetest.add_particlespawner({
			amount = 256,
			time = 1,
			minpos = vector.subtract(pos,{x=0.5,y=0,z=0.5}),
			maxpos = vector.add(pos,{x=0.5,y=2,z=0.5}),
			minvel = {x=-1, y=-2, z=-1},
			maxvel = {x=1,  y=1,  z=1},
			minacc = vector.new(),
			maxacc = vector.new(),
			minexptime = 0.5,
			maxexptime = 1,
			minsize = 1,
			maxsize = 2,
			physical = true,
			texture = "11.png",
		})
	pos.y = pos.y + 0.5
	drop(itemstack)

end
function antiprop(max,maxarg,arg)
	return (max*maxarg)/math.max(arg,maxarg)
end
function db(message)
	if type(message) ~= string then
		message = tostring(message)
	end
	minetest.chat_send_all(message)
end
set_an_animation = function(self, t)
	if t == 1 and self.canimation ~= 1 then
		self.object:set_animation({
			x = 0,
			y = 250},
			30, 0)
		self.canimation = 1
	elseif t == 2 and self.canimation ~= 2 then
		self.object:set_animation({x = 0,y = 250},60, 0)
		self.canimation = 2
	end
end
function mod(n1,n2)
	return n1-math.floor(n1/n2)*n2
end
function plainddv(v1,v2)
	local v1 = vector.normalize(v1)
	local v2 = vector.normalize(v2)
	local xd = v1.x-v2.x
	local yd = v1.z-v2.z
	return xd*xd+yd*yd
end
function plaindirectiondifference(v1,v2)
	return math.abs(mod(math.atan2(v1.z,v1.x)-math.atan(v2.z,v2.x),2*math.pi)-math.pi)
end
function short(vektor,lenght)
	return vector.multiply(vector.normalize(vektor),lenght)
end
function plainvector(yaw)
	return {x = math.cos(yaw),y=0,z = math.sin(-yaw)}
end
function rotate(vektor,yaw)
	returning = plainvector(math.atan2(vektor.z,vektor.x)+yaw)
	returning.y = vektor.y
	return returning
end
function rotatef(vektor,yaw)
	return {x = vektor.x * math.cos(yaw) - vektor.z * math.sin(yaw),y = vektor.y,z = vektor.x * math.sin(yaw) + vektor.z * math.cos(yaw)}
end
function register_spawn(def)
		def.neighbors = def.neighbors or "air" 
		def.height = def.height or {min = -1000000000,max = 1000000000}
		def.timeofday = def.timeofday or {min = 0,max = 1}
		def.max_count = def.maxcount or 3000
		def.light = def.light or {min = -1000000,max = 1000000}
	minetest.register_abm({
		nodenames = def.nodes,
		neighbors = def.neighbors,
		interval = creepers_spawn_constant,
		chance = def.chanche,
		action = function(pos)
			pos.y = pos.y+1
			local ll = minetest.get_node_light(pos) or 0
			local dt = minetest.get_timeofday()
			if minetest.get_node(pos).name ~= "air"then return 
elseif dt < def.timeofday.max and
 dt > def.timeofday.min and
 ll < def.light.max and
 ll > def.light.min and
 pos.y < def.height.max and
 pos.y > def.height.min and
 len(minetest.get_objects_inside_radius(pos,25)) < def.max_count then
			minetest.add_entity(pos, "creepers:" .. def.name)
			end
		end
	})
end
minetest.register_craftitem("creepers:jelly", {
	description = "Jelly",
	inventory_image = "jelly.png",
	on_use = minetest.item_eat(10),
})
function register_creeper_advanced(name, def)
	local defbox = def.size/8
	local side2 = name .. "3.png"
	local side = name .. "2.png"
	local front = name .. "1.png"
	minetest.register_entity("creepers:" .. name,{
		initial_properties = {
			name = "creepers:" .. name,
			hp_max = def.max_hp,
			visual_size = {x = def.size*2, y = def.size*2, z = def.size*2},
			visual = "mesh",
			mesh = "creeper_rotated.b3d",
			textures = {def.textures .. ".png^cm.png"},
			collisionbox = {-defbox,-0.1 , -defbox, defbox, def.size, defbox},
			physical = true
		},
		-- ON ACTIVATE --
		on_punch = function(self)
			dropondie(self,def.drops)
		end,
		on_activate = function(self)
			self.jump = 0
			self.flushed = 0
			self.charging_radius = def.charging_radius
			self.charge = 0
			self.charging_time = def.charging_time
			self.targetvektor = {x=math.random(-1,1)*def.speed, y = 0 , z = math.random(-1,1)*def.speed }
			self.object:setacceleration({x = 0, y = -500, z = 0})
			self.object:setvelocity({x = math.random(-1,1), y = math.random(-1,1), z = math.random(-1,1)})
			self.timer = 0
			self.object:set_animation({
				x = 0,
				y = 250},
				30, 0)
			self.canimation = 1
		end,
		-- ON PUNCH --
		-- ON STEP --
		on_step = function(self, dtime)
			if self.timer > 3600 and def.lifetime == nil then
				self.timer = 0
			elseif def.lifetime ~= nil and self.timer > def.lifetime then
				self.object:remove()
			end
			self.timer = self.timer + dtime
			local pos = self.object:getpos()
			if not(self.object:get_hp() > 0) then
				dropondie(self,def.drops)
			end
			if math.random(1,50) == 1 then
				self.targetvektor = {x=math.random(-10,10),y=0,z=math.random(-10,10)}
				self.targetvektor = short(self.targetvektor,def.speed)
			end
			self.attack = nil
			for _,entity in ipairs(minetest.get_objects_inside_radius(pos,def.vievrange)) do
				if entity:is_player() then
					self.attack = true
					self.targetvektor = short(vector.subtract(entity:getpos(),pos),def.speed)
					if plainddv(vector.subtract(vector.new(),self.targetvektor),entity:get_look_dir())<1 then
						self.targetvektor = rotatef(self.targetvektor,math.sin(self.timer*math.pi/5)*math.pi/2)
					end
				end
			end
			if self.attack then
				set_an_animation(self,2)
			else
				set_an_animation(self,1)
			end
			local velocity = self.object:getvelocity()
			self.jump = (self.jump +1)%10
			if vector.distance(vector.new(),velocity)<def.speed/10 and velocity.y == 0 and self.jump == 0 then
				pos.y = pos.y + 2
				self.object:setpos(pos)
			end
			if def.creeperking and self.timer%def.reload < 0.1 then
				minetest.add_entity(self.object:getpos(),def.soldiers)
			end
			if def.regen then
				self.object:set_hp(self.object:get_hp()+def.regen)
			end
			if self.flushed > 0 then
				self.flushed = self.flushed-1
				if self.flushed == 0 then
					self.object:set_hp(self.last_hp)
				end
			end
			for _,obj in ipairs(minetest.get_objects_inside_radius(pos,self.charging_radius)) do
				if obj:is_player() then
					if self.flushed == 0 then
					self.charge = self.charge+1
					if self.charge > self.charging_time*creepers_charging_constant then
						if def.flusharmor then
							self.object:set_hp(1000000000)
							self.flushed = 20
							self.last_hp = self.object:get_hp()
						end
						creepers_explodeadvanced({x=math.floor(pos.x),y=math.floor(pos.y),z=math.floor(pos.z)},def.radius,def.damage)
						self.charge = 0
					end
					end
				end
			end
			if self.targetvektor then
				self.object:setvelocity(self.targetvektor)
				--self.object:set_properties({automatic_rotate = 0})
				self.object:setyaw(math.atan2(self.targetvektor.z,self.targetvektor.x))
			end
		end
		--db(self.canimation)
	})
	register_spawn({
		light = def.light,
		height = def.height,
		timeofday = def.timeofday,
		max_count = def.max_count,
		neighbors = def.neighbors,
		chanche = def.chanche,
		nodes = def.spawn,
		name = name
		})
end
local w_stuff = {"default:desert_sand"}
local c_stuff = {"group:stone","nether:rack","nether:sand","default:gravel"}
local i_stuff = {"default:snow","default:snowblock","default:ice"}
local a_stuff = {"group:stone","nether:rack","nether:sand","default:gravel","group:soil","group:sand","default:snow","default:snowblock","default:ice","group:snappy"}
local s_stuff = {"group:soil","group:sand","default:snow","default:snowblock","default:ice","group:snappy"}
local d_stuff = {"default:mossycobble","nether:brick"}
local big_t = {{name = "default:mese",chanche = 5,min = 1,max = 10},{name = "default:diamondblock",chanche = 5,min = 1,max = 10},{name = "default:goldblock",chanche = 5,min = 1,max = 10}}
local medium_t = {{name = "default:mese_crystal",chanche = 25,min = 1,max = 10},{name = "default:diamond",chanche = 25,min = 1,max = 10},{name = "default:gold_ingot",chanche = 25,min = 1,max = 10},{name = "default:mese",chanche = 100,min = 1,max = 10},{name = "default:diamondblock",chanche = 100,min = 1,max = 10},{name = "default:goldblock",chanche = 100,min = 1,max = 10}}
local small_t = {{name = "default:coalblock",chanche = 5,min = 1,max = 10},{name = "default:steel_ingot",chanche = 10,min = 1,max = 10},{name = "default:mese_crystal",chanche = 250,min = 1,max = 10},{name = "default:diamond",chanche = 250,min = 1,max = 10},{name = "default:gold_ingot",chanche = 250,min = 1,max = 10},{name = "default:mese",chanche = 1000,min = 1,max = 10},{name = "default:diamondblock",chanche = 1000,min = 1,max = 10},{name = "default:goldblock",chanche = 1000,min = 1,max = 10}}
register_creeper_advanced("swat_soldier", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	height = {min = -1000000000,max = -1024},
	max_count = 100,
	textures = "c17",
	lifetime = 100000,
	size = 3,
	speed = 4,
	max_hp = 250,
	damage = 10,
	radius = 12,
	charging_radius = 10,
	charging_time = 2,
	drops = medium_t,
	chanche = 3000/creepers_cave_boni,
	vievrange = 65,
	spawn = c_stuff,
})
register_creeper_advanced("elite_soldier", {
	light = {min = 0,max = 2},
	dtime = {min = 0.8,max = 1},
	height = {min = 0.75,max = 1},
	max_count = 100,
	textures = "c18",
	lifetime = 50000,
	size = 3,
	speed = 3,
	max_hp = 150,
	damage = 8,
	radius = 9,
	charging_radius = 8,
	charging_time = 6,
	drops = medium_t,
	chanche = 2000/creepers_cave_boni,
	vievrange = 45,
	spawn = c_stuff,
})
register_creeper_advanced("a_level_soldier", {
	light = {min = 0,max = 4},
	dtime = {min = 0.8,max = 1},
	height = {min = 0.75,max = 1},
	max_count = 100,
	textures = "c20",
	lifetime = 25000,
	size = 3,
	speed = 3,
	max_hp = 100,
	damage = 6,
	radius = 7,
	charging_radius = 5,
	charging_time = 8,
	drops = small_t,
	chanche = 1500/creepers_cave_boni,
	vievrange = 35,
	spawn = c_stuff,
})
register_creeper_advanced("bone_soldier", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c16",
	lifetime = 10000,
	size = 2,
	speed = 2,
	max_hp = 50,
	damage = 5,
	radius = 5,
	charging_radius = 3,
	charging_time = 10,
	drops = small_t,
	chanche = 1000/creepers_cave_boni,
	vievrange = 25,
	spawn = c_stuff,
})
register_creeper_advanced("mountain_horror", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	height = {min = 0,max = 1000000000},
	max_count = 75,
	textures = "c16",
	size = 3,
	speed = 1,
	creeperking = true,
	reload = 10,
	soldiers = "creepers:bone_soldier",
	flusharmor = 1000,
	max_hp = 750,
	damage = 25,
	radius = 20,
	charging_radius = 4,
	charging_time = 10,
	drops = medium_t,
	chanche = 60000/creepers_cave_boni,
	vievrange = 75,
	spawn = c_stuff,
})
register_creeper_advanced("cave_horror", {
	light = {min = 0,max = 4},
	dtime = {min = 0.8,max = 1},
	height = {min = -128,max = 0},
	max_count = 75,
	textures = "c16",
	size = 4,
	speed = 1,
	creeperking = true,
	reload = 10,
	soldiers = "creepers:bone_soldier",
	flusharmor = 1000,
	max_hp = 1250,
	damage = 25,
	radius = 30,
	charging_radius = 5,
	charging_time = 9,
	drops = medium_t,
	chanche = 50000/creepers_cave_boni,
	vievrange = 100,
	spawn = c_stuff,
})
register_creeper_advanced("miners_death", {
	light = {min = 0,max = 3},
	dtime = {min = 0.75,max = 1},
	height = {min = -1024,max = -128},
	max_count = 75,
	textures = "c15",
	size = 5,
	speed = 1,
	creeperking = true,
	reload = 5,
	soldiers = "creepers:a_level_soldier",
	flusharmor = 1000,
	max_hp = 2500,
	damage = 50,
	radius = 40,
	charging_radius = 6,
	charging_time = 7,
	drops = medium_t,
	chanche = 60000/creepers_cave_boni,
	vievrange = 100,
	spawn = c_stuff,
})
register_creeper_advanced("king_of_darkness", {
	light = {min = 0,max = 2},
	dtime = {min = 0.75,max = 1},
	height = {min = -5000,max = -1024},
	max_count = 75,
	textures = "c18",
	size = 6,
	speed = 2,
	creeperking = true,
	reload = 3,
	soldiers = "creepers:elite_soldier",
	flusharmor = 10000,
	max_hp = 5000,
	damage = 100,
	radius = 50,
	charging_radius = 7,
	charging_time = 5,
	drops = big_t,
	chanche = 90000/creepers_cave_boni,
	vievrange = 150,
	spawn = c_stuff,
})
register_creeper_advanced("netherking", {
	light = {min = 0,max = 10},
	dtime = {min = 0.75,max = 1},
	height = {min = -1000000000,max = -5000},
	max_count = 50,
	textures = "c17",
	size = 8,
	speed = 3,
	creeperking = true,
	reload = 2,
	soldiers = "creepers:swat_soldier",
	flusharmor = 1000000,
	max_hp = 10000,
	damage = 250,
	radius = 80,
	charging_radius = 9,
	charging_time = 3,
	drops = big_t,
	chanche = 120000/creepers_cave_boni,
	vievrange = 200,
	spawn = c_stuff,
})
register_creeper_advanced("dungeon_king", {
	light = {min = 0,max = 3},
	dtime = {min = 0.75,max = 1},
	height = {min = -1000000000,max = 10000000000},
	max_count = 75,
	textures = "c7",
	size = 5,
	speed = 1,
	creeperking = true,
	reload = 5,
	soldiers = "creepers:a_level_soldier",
	flusharmor = 1000,
	max_hp = 4000,
	damage = 60,
	radius = 50,
	charging_radius = 6,
	charging_time = 6,
	drops = medium_t,
	chanche = 60000/creepers_cave_boni,
	vievrange = 100,
	spawn = d_stuff,
})
register_creeper_advanced("dungeon_soldier", {
	light = {min = 0,max = 4},
	dtime = {min = 0.75,max = 1},
	height = {min = -1000000000,max = 10000000000},
	max_count = 100,
	textures = "c7",
	lifetime = 30000,
	size = 3,
	speed = 3,
	max_hp = 125,
	damage = 8,
	radius = 8,
	charging_radius = 4,
	charging_time = 8,
	drops = small_t,
	chanche = 1500/creepers_cave_boni,
	vievrange = 35,
	spawn = c_stuff,
})
register_creeper_advanced("surface_soldier", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c5",
	lifetime = 10000,
	size = 2,
	speed = 2,
	max_hp = 25,
	damage = 3,
	radius = 3,
	charging_radius = 3,
	charging_time = 15,
	drops = small_t,
	chanche = 1000,
	vievrange = 20,
	spawn = s_stuff,
})
register_creeper_advanced("creeper", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c21",
	lifetime = 8000,
	size = 2,
	speed = 2,
	max_hp = 20,
	damage = 2,
	radius = 3,
	charging_radius = 2,
	charging_time = 18,
	drops = small_t,
	chanche = 1000,
	vievrange = 15,
	spawn = a_stuff,
})
register_creeper_advanced("grass_soldier", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c5",
	lifetime = 10000,
	size = 2,
	speed = 2,
	max_hp = 28,
	damage = 3,
	radius = 4,
	charging_radius = 4,
	charging_time = 14,
	drops = small_t,
	chanche = 1000,
	vievrange = 22,
	spawn = s_stuff,
})
register_creeper_advanced("icy_soldier", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c6",
	lifetime = 12000,
	size = 2,
	speed = 2,
	max_hp = 30,
	damage = 4,
	radius = 5,
	charging_radius = 3,
	charging_time = 18,
	drops = small_t,
	chanche = 1000,
	vievrange = 20,
	spawn = i_stuff,
})
register_creeper_advanced("desert_soldier", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c12",
	lifetime = 10000,
	size = 2,
	speed = 2,
	max_hp = 25,
	damage = 3,
	radius = 3,
	charging_radius = 4,
	charging_time = 12,
	drops = small_t,
	chanche = 1000,
	vievrange = 25,
	spawn = w_stuff,
})
register_creeper_advanced("stone_soldier", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c19",
	lifetime = 12000,
	size = 2,
	speed = 2,
	max_hp = 30,
	damage = 5,
	radius = 6,
	charging_radius = 4,
	charging_time = 12,
	drops = small_t,
	chanche = 1000/creepers_cave_boni,
	vievrange = 25,
	spawn = c_stuff,
})
register_creeper_advanced("joker", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c13",
	lifetime = 10000,
	size = 2,
	speed = 1,
	max_hp = 10,
	damage = 15,
	radius = 15,
	charging_radius = 2,
	charging_time = 50,
	drops = small_t,
	chanche = 240000,
	vievrange = 10,
	spawn = a_stuff,
})
register_creeper_advanced("lauch", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 100,
	textures = "c14",
	lifetime = 10000,
	size = 2,
	speed = 3,
	max_hp = 10,
	damage = 1,
	radius = 3,
	charging_radius = 2,
	charging_time = 15,
	drops = small_t,
	chanche = 240000,
	vievrange = 25,
	spawn = a_stuff,
})
register_creeper_advanced("surface_king", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 50,
	textures = "c5",
	size = 4,
	speed = 1,
	creeperking = true,
	reload = 15,
	soldiers = "creepers:surface_soldier",
	flusharmor = 1000,
	max_hp = 250,
	damage = 15,
	radius = 15,
	charging_radius = 3,
	charging_time = 40,
	drops = small_t,
	chanche = 40000,
	vievrange = 40,
	spawn = s_stuff,
})
register_creeper_advanced("grass_king", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 50,
	textures = "c5",
	size = 4,
	speed = 1,
	creeperking = true,
	reload = 12,
	soldiers = "creepers:grass_soldier",
	flusharmor = 1000,
	max_hp = 300,
	damage = 16,
	radius = 18,
	charging_radius = 4,
	charging_time = 35,
	drops = small_t,
	chanche = 40000,
	vievrange = 43,
	spawn = s_stuff,
})
register_creeper_advanced("desert_king", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 50,
	textures = "c5",
	size = 4,
	speed = 1,
	creeperking = true,
	reload = 10,
	soldiers = "creepers:desert_soldier",
	flusharmor = 1000,
	max_hp = 250,
	damage = 10,
	radius = 8,
	charging_radius = 5,
	charging_time = 25,
	drops = small_t,
	chanche = 40000,
	vievrange = 45,
	spawn = w_stuff,
})
register_creeper_advanced("icy_king", {
	light = {min = 0,max = 10},
	dtime = {min = 0.8,max = 1},
	max_count = 50,
	textures = "c5",
	size = 4,
	speed = 1,
	creeperking = true,
	reload = 15,
	soldiers = "creepers:icy_soldier",
	flusharmor = 1000,
	max_hp = 350,
	damage = 18,
	radius = 20,
	charging_radius = 2,
	charging_time = 45,
	drops = small_t,
	chanche = 40000,
	vievrange = 30,
	spawn = i_stuff,
})
register_creeper_advanced("stone_king", {
	light = {min = 0,max = 10},
	dtime = {min = 0.75,max = 1},
	max_count = 50,
	textures = "c19",
	size = 4,
	speed = 1,
	creeperking = true,
	reload = 10,
	soldiers = "creepers:stone_soldier",
	flusharmor = 1000,
	max_hp = 300,
	damage = 20,
	radius = 20,
	charging_radius = 4,
	charging_time = 30,
	drops = small_t,
	chanche = 40000/creepers_cave_boni,
	vievrange = 50,
	spawn = _stuff,
})
--[[
register_creeper_advanced("king", {
	light = {min = 0,max = 14},
	dtime = {min = 0,max = 14},
	max_count = 600,
	neighbors = {"air"},
	textures = "c16",
	size = 4,
	speed = 1,
	creeperking = true,
	reload = 10,
	soldiers = "creepers:test",
	flusharmor = 1000,
	max_hp = 1,
	damage = 10,
	radius = 5,
	charging_radius = 2,
	charging_time = 100,
	drops = {{name = "creepers:jelly",chanche = 1,min = 1,max = 3}},
	chanche = 100,
	vievrange = 25,
	spawn = "default:stone",
})
]]
function register_creeper(def)
	register_creeper_advanced(def.name, {
		size = def.size,
		speed = 2 or def.speed,
		max_hp = 500 or def.max_hp,
		damage = def.damage,
		drops = "" or {{name = "creepers:jelly",chanche = 1,min = 1,max = 3}},
		chanche = 10000 or def.chanche,
		spawn = def.spawn,
	})
end
--[[register_creepers({
	name = "a",
	size = 0.25,
	max_hp = 10,
	damage = 5,
	chanche = 80000,
	speed = 3
})
register_creepers({
	name = "b",
	size = 0.3,
	max_hp = 15,
	damage = 4,
	chanche = 40000
})
register_creepers({
	name = "c",
	size = 0.5,
	max_hp = 25,
	damage = 2,
	chanche = 20000
})
register_creepers({
	name = "d",
	size = 0.25,
	max_hp = 10,
	damage = 1,
	chanche = 20000
})
register_creepers({
	name = "e",
	size = 0.2,
	max_hp = 10,
	damage = 10,
	chanche = 80000,
	speed = 1.5
})
register_creepers({
	name = "f",
	size = 0.1,
	max_hp = 5,
	damage = 8,
	chanche = 40000,
	speed = 1
})
register_creepers({
	name = "g",
	size = 0.75,
	max_hp = 20,
	damage = 2,
	chanche = 20000,
	speed = 2.5
})
register_creepers({
	name = "h",
	size = 0.75,
	max_hp = 20,
	damage = 5,
	chanche = 80000,
	speed = 3
})]]
